package com.yunzhi.fifteenpuzzle.common;

import com.alipay.api.*;
import com.alipay.api.request.AlipayTradePagePayRequest;
import com.alipay.api.response.AlipayTradePagePayResponse;
import com.yunzhi.fifteenpuzzle.config.AliPayConfig;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

import java.io.IOException;

@Component
public class AliPayUtil {

    AliPayConfig config;
    AlipayClient client;

    @Value("${yz.servBase}")
    String servBase;

    @Autowired
    public AliPayUtil(AliPayConfig config) {
        this.config = config;

        try {
            initClient();
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    public AlipayClient getClient() {
        if (client == null) {
            try {
                return initClient();
            } catch (IOException e) {
                e.printStackTrace();
            }
        }

        return client;
    }

    private AlipayClient initClient() throws IOException {
        AlipayConfig alipayConfig = new AlipayConfig();
        BeanUtils.copyProperties(config, alipayConfig);

        // privateKey 其实是文件
        String privateKey = StringUtils.readFrom(this.config.getPrivateKey());
        alipayConfig.setPrivateKey(privateKey);

        try {
            client = new DefaultAlipayClient(alipayConfig);
            return client;
        } catch (AlipayApiException e) {
            e.printStackTrace();
        }

        return null;
    }

    public AlipayTradePagePayResponse pagePay(AlipayObject model, String orderId, String url) throws AlipayApiException {
        AlipayTradePagePayRequest request = new AlipayTradePagePayRequest();
        request.setNotifyUrl(servBase + "/alipay/notify/" + orderId);
        request.setReturnUrl(url);
        request.setBizModel(model);
        return client.pageExecute(request);
    }
}
