package com.yunzhi.fifteenpuzzle.common;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

public class Ida {

    public final int N = 4;
    public final int N2 = 16;
    public final int LIMIT = 120;
    private final int[] dx = { 0, -1, 0, 1 };
    private final int[] dy = { 1, 0, -1, 0 };
    private final String[] dir = { "←", "↓", "→", "↑" };
    private static int[][] MDT; // 16 * 16 曼哈顿距离
    private int limit;
    private Map<Integer, Integer> path;
    private Puzzle state;

    public Ida() {
        initMDT();
    }

    private void initMDT() {
        MDT = new int[N2][N2];

        for (int i = 0; i < N2; i++) {
            int[] tmp = new int[N2];
            for (int j = 0; j < N2; j++) {
                int m = Math.abs(i / N - j / N);
                int n = Math.abs(i % N - j % N);
                tmp[j] = m + n;
            }
            MDT[i] = tmp;
        }
    }

    private void reset() {
        limit = 0;
        path = new HashMap<>();
        state = null;
    }

    private void swap(int[] f, int i, int j) {
        int tmp = f[i];
        f[i] = f[j];
        f[j] = tmp;
    }

    private int getAllMD(Puzzle pz) {
        int sum = 0;
        for (int i = 0; i < N2; i++) {
            int p = pz.f[i];
            if (p == N2) continue;
            sum += MDT[i][p - 1];   // p - 1 因为是 0 - 15 不是 1 -16
        }
        return sum;
    }

    private boolean dfs(int depth, int prev) {
        if (state.MD == 0) return true;
        if(depth + state.MD > limit) return false;

        int sx = state.space / N;
        int sy = state.space % N;

        Puzzle tmp;

        for (int r = 0; r < 4; r ++) {
            int tx = sx + dx[r];
            int ty = sy + dy[r];

            if(tx < 0 || ty < 0 || tx >= N || ty >= N ) continue;

            // 防止回溯
            if(Math.max(prev, r) - Math.min(prev, r) == 2) continue;

            tmp = state.clone();

            //计算曼哈顿距离的差值，同时交换拼图块
            state.MD -= MDT[tx * N + ty][state.f[tx * N + ty] - 1];
            state.MD += MDT[sx * N + sy][state.f[tx * N + ty] - 1];
            swap(state.f, tx * N + ty, sx * N + sy);
            state.space = tx * N + ty;

            if (dfs(depth + 1, r)) {
                path.put(depth, r);
                return true;
            }

            state = tmp;
        }

        return false;
    }

    public List<String> calc(Puzzle pz) throws Exception {
        reset();

        pz.MD = getAllMD(pz);
        for (limit = pz.MD; limit <= LIMIT; limit++) {
            state = pz.clone();
            if( dfs(0, -100) ) {
                List<String> res = new ArrayList<>();

                for (int i = 0; i < limit; i++) {
                    res.add(dir[path.get(i)]);
                }

                return res;
            }
        }

        throw new Exception("unsolvable");
    }

    public static class Puzzle {
        public int[] f;
        public int space;
        public int MD;

        public Puzzle(int[] f, int space) {
            this.f = f;
            this.space = space;
        }

        private int[] copyF() {
            int length = f.length;
            int[] f2 = new int[length];
            for (int i = 0; i < length; i++) {
                f2[i] = f[i];
            }
            return f2;
        }

        public Puzzle clone() {
            int[] f2 = copyF();
            Puzzle copy = new Puzzle(f2, space);
            copy.MD = MD;
            return copy;
        }
    }
}
